//=============================================================================

#include <cmath>
#include <iostream>
#include <iomanip>
#include <string_view>
#include <optional>
using namespace std;

//-----------------------------------------------------------------------------

optional<int> readUserInput(string_view msg)
{
   cout << msg << ": ";
   int value;
   cin >> value;
   if (cin.fail())
   {
      cout << "Fehlerhafte Eingabe\n";
      return {};
   }
   return value;
}

string::size_type calcLengthOfMaxMultiplicationValue(const int maxLimit)
{
   const int maxValue = maxLimit * maxLimit;
   const string::size_type digits = 1 + static_cast<string::size_type>(log10(maxValue));
   return digits;
}

void printTitleLine(const string::size_type digits, const int minLimit, const int maxLimit)
{
   const string::size_type spaceCount = digits + 2;
   const string spaces(spaceCount, ' ');
   cout << spaces << '|';
   for (int i = minLimit; i <= maxLimit; ++i)
   {
      cout << setw(spaceCount) << i;
   }
   cout << '\n';
}

void printSeperatorLine(const string::size_type digits, const int minLimit, const int maxLimit)
{
   const string::size_type count = (digits + 4) + (maxLimit - minLimit + 1) * (2 + digits);
   string seperator(count, '-');
   seperator[digits + 2] = '+';
   cout << seperator << '\n';
}

void printMatrix(const string::size_type digits, const int minLimit, const int maxLimit)
{
   for (int i = minLimit; i <= maxLimit; ++i)
   {
      cout << ' ' << setw(digits) << i << " |";
      for (int j = minLimit; j <= maxLimit; ++j)
      {
         cout << "  " << setw(digits) << (i * j);
      }
      cout << '\n';
   }
}

int main()
{
   cout << "WS2024_04\n\n";

   const auto minResult = readUserInput("Untere Grenze");
   if (!minResult) return 1;
   const auto minLimit = *minResult;

   const auto maxResult = readUserInput("Obere Grenze");
   if (!maxResult) return 1;
   const auto maxLimit = *maxResult;

   if (minLimit > maxLimit)
   {
      cout << "Fehlerhafte Eingabe: untere Grenze ist groesser als obere Grenze\n";
      return 1;
   }

   if (maxLimit - minLimit > (12-1))
   {
      cout << "Fehlerhafte Eingabe: zu viele Eintraege, max. 12 unterstuetzt\n";
      return 1;
   }

   const auto digits = calcLengthOfMaxMultiplicationValue(maxLimit);

   cout << '\n';
   printTitleLine(digits, minLimit, maxLimit);
   printSeperatorLine(digits, minLimit, maxLimit);
   printMatrix(digits, minLimit, maxLimit);
}
