//=============================================================================

#define CATCH_CONFIG_MAIN
#include "catch.hpp"
#include <sstream>
#include <string>
using namespace std;

//-----------------------------------------------------------------------------
// cout ist ein Ausgabe-Strom - ostream
// cin ist ein Eingabe-Strom - istream

string readUserName(istream& in, ostream& out)
{
   for (;;)
   {
      out << "Name: ";
      string name;
      getline(in, name);
      if (!name.empty())
      {
         return name;
      }
      out << "Nicht nichts\n";
   }
}

void hello(ostream& out, string_view name)
{
   const string::size_type starwidth = 11 + name.length();
   const string stars(starwidth, '*');

   const string::size_type spacewidth = starwidth - 2;
   const string spaces(spacewidth, ' ');

   out << stars << "\n"
      <<'*' << spaces << "*\n"
      << "* Hallo " << name << "! *\n"
      << '*' << spaces << "*\n"
      << stars << '\n';
}

void mymain(istream& in, ostream& out)
{
   string name = readUserName(in, out);
   out << '\n';
   hello(out, name);
}

//-----------------------------------------------------------------------------

TEST_CASE("Name einlesen - einfacher Fall")
{
   istringstream iss("Tom");
   ostringstream oss;
   REQUIRE(readUserName(iss, oss) == "Tom");
   REQUIRE(oss.str() == "Name: ");
}

TEST_CASE("Name einlesen - komplexer Fall")
{
   istringstream iss("Tom Mustermann");
   ostringstream oss;
   REQUIRE(readUserName(iss, oss) == "Tom Mustermann");
   REQUIRE(oss.str() == "Name: ");
}

TEST_CASE("Name einlesen - widerborstiger User")
{
   istringstream iss("\nMax");
   ostringstream oss;
   REQUIRE(readUserName(iss, oss) == "Max");
   REQUIRE(oss.str() == "Name: Nicht nichts\nName: ");
}

TEST_CASE("Name einlesen - sehr widerborstiger User")
{
   istringstream iss("\n\n ");
   ostringstream oss;
   REQUIRE(readUserName(iss, oss) == " ");
   REQUIRE(oss.str() == "Name: Nicht nichts\nName: Nicht nichts\nName: ");
}

TEST_CASE("Hallo - mit einfachem Namen")
{
   ostringstream oss;
   hello(oss, "Tom");
   REQUIRE(oss.str() == "**************\n*            *\n* Hallo Tom! *\n*            *\n**************\n");
}

TEST_CASE("Hallo - mit einem Buchstaben")
{
   ostringstream oss;
   hello(oss, " ");
   REQUIRE(oss.str() == "************\n*          *\n* Hallo  ! *\n*          *\n************\n");
}

TEST_CASE("Hallo - mit komplexem Namen")
{
   ostringstream oss;
   hello(oss, "Max Muster");
   REQUIRE(oss.str() == "*********************\n*                   *\n* Hallo Max Muster! *\n*                   *\n*********************\n");
}

TEST_CASE("Hauptprogramm")
{
   istringstream iss("Eva");
   ostringstream oss;
   mymain(iss, oss);
   REQUIRE(oss.str() == "Name: \n**************\n*            *\n* Hallo Eva! *\n*            *\n**************\n");
}
